// Simple analog in/out program that reads the input voltage and passes
// it to the dac.  Uses PA0 for analog in, PA4 for analog out.  Be
// careful with labels here as they differ from the labels on the board.
// This program uses labels as defined in the chip datasheet/reference manual.
#include "stm32f303xx.h"
#include "serial.h"
void delay(int);
volatile unsigned SysTickCounter=0;
volatile unsigned Seconds=0;
void initADC(void);
void initDAC(void);
int readADC(void);
void writeDAC(int Value);

// Define the sample rate here (in Hz).  Using this mechanism we 
// limited to about 100kHz
#define Fs 100000
void delay(int dly)
{
  while( dly--);
}
void initSysTick()
{
        
	STK_CSR |= ( BIT2 | BIT1 | BIT0); // enable systick, source = cpu clock, enable interrupt
// SysTick clock source = 64MHz.  Divide this down to create sample period
	STK_RVR = (64000000/Fs);   
	STK_CVR = 10; // don't want long wait for counter to count down from initial high unknown value
	SysTickCounter = 0;        
}
void SysTick()
{
// This should occur at a rate of NHz.
	GPIOB_ODR |= BIT3;
	SysTickCounter++;	
	writeDAC(readADC());
	GPIOB_ODR &= ~BIT3;
}
void configPins()
{
	// Power up PORTB
	RCC_AHBENR |= BIT18;
	GPIOB_MODER |= BIT6; // make bit3  an output
	GPIOB_MODER &= ~BIT7; // make bit3  an output

}
void initADC()
{
	
	// Power up PORTA
	RCC_AHBENR |= BIT17;
	GPIOA_MODER |= BIT1+BIT0; // put PA0 into analog mode
	// Will be using PA0 as an analog input which corresponds
	// to ADC1_IN1
	RCC_AHBENR |= BIT28; // turn on ADC clock
	// Set ADC clock = HCLK / 2=32MHz
	ADC12_CCR |= BIT17;
	// disable the ADC
	ADC1_CR &= ~BIT0;	
	// enable the ADC voltage regulator
	ADC1_CR &= ~BIT29;
	ADC1_CR |= BIT28;
	// start ADC calibration cycle
	ADC1_CR |= BIT31; 
	// wait for calibration to complete
	while (ADC1_CR & BIT31);
	
	// enable the ADC
	ADC1_CR |= BIT0;	
	
	// Select ADC Channel 1
	ADC1_SQR1 = (1 << 6);
}

int readADC()
{
	ADC1_CR |= BIT2; // start conversion
	while (ADC1_CR & BIT2); // wait for end of conversion
	return ADC1_DR;
}
void initDAC()
{
	// Will be using DAC1_OUT1 on PA4
	// Power up PORTA
	RCC_AHBENR |= BIT17;
	// Select Analog mode for PA4
	GPIOA_MODER |= BIT9+BIT8;
	// enable the DAC clock
	RCC_APB1ENR |= BIT29;
	DAC_CR |= BIT0; // enable DAC1;
}
void writeDAC(int Value)
{
	DAC_DHR12R1 = Value;
	
}
void initClock()
{
	// Assumption: Using HSI clock (8MHz)
	// This clock is provided by the STLink debug interface
	// PLL is driven by HSI/2 = 4MHz
	// Multiply x 16  = 64MHz
	
	// Must set flash latency (2 wait states) to allow
	// for higher speed clock	
	FLASH_ACR |= 2; // add two wait states
	
	RCC_CR &= ~BIT24; // turn off PLL
	delay(10);	
	RCC_CFGR |= ((0xf) << 18);	// PLL multiply = 9	
	RCC_CFGR |= BIT10;  // APB1 Low speed Clock = HCLK/2 (32MHz)	
	delay(10);
	RCC_CR |= BIT24; // turn on PLL
	delay(10);
	RCC_CFGR |= BIT1;  // switch to PLL as clock
}
int main()
{	
	char buf[10];
	initClock();
	configPins();
	initUART(9600);
	initSysTick();
	initADC();
	initDAC();
	unsigned count=0;
	while(1)
	{			
	}
	return 0;
}








